
/* Chrysalide - Outil d'analyse de fichiers binaires
 * literal.c - manipulation des valeurs littérales de types instanciés
 *
 * Copyright (C) 2012-2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "literal.h"


#include <malloc.h>
#include <stdio.h>
#include <string.h>


#include "basic.h"
#include "../type-int.h"



/* Description de type instancié avec une valeur litérale (instance) */
struct _GLiteralType
{
    GDataType parent;                       /* A laisser en premier        */

    GDataType *orig;                        /* Type instancié              */
    literal_value value;                    /* Valeur d'instance           */

};

/* Description de type instancié avec une valeur litérale (classe) */
struct _GLiteralTypeClass
{
    GDataTypeClass parent;                  /* A laisser en premier        */

};


/* Initialise la classe des types instanciés avec des valeurs. */
static void g_literal_type_class_init(GLiteralTypeClass *);

/* Initialise l'instance d'un type instancié avec une valeur. */
static void g_literal_type_init(GLiteralType *);

/* Supprime toutes les références externes. */
static void g_literal_type_dispose(GLiteralType *);

/* Procède à la libération totale de la mémoire. */
static void g_literal_type_finalize(GLiteralType *);

/* Crée un copie d'un type existant. */
static GDataType *g_literal_type_dup(const GLiteralType *);

/* Décrit le type fourni sous forme de caractères. */
static char *g_literal_type_to_string(const GLiteralType *, bool);



/* Indique le type défini pour un type reposant sur des gabarits. */
G_DEFINE_TYPE(GLiteralType, g_literal_type, G_TYPE_DATA_TYPE);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des types instanciés avec des valeurs.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_literal_type_class_init(GLiteralTypeClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GDataTypeClass *type;                   /* Version parente et basique  */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_literal_type_dispose;
    object->finalize = (GObjectFinalizeFunc)g_literal_type_finalize;

    type = G_DATA_TYPE_CLASS(klass);

    type->dup = (type_dup_fc)g_literal_type_dup;
    type->to_string = (type_to_string_fc)g_literal_type_to_string;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise l'instance d'un type instancié avec une valeur.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_literal_type_init(GLiteralType *type)
{
    type->orig = NULL;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_literal_type_dispose(GLiteralType *type)
{
    if (type->orig != NULL)
        g_object_unref(G_OBJECT(type->orig));

    G_OBJECT_CLASS(g_literal_type_parent_class)->dispose(G_OBJECT(type));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_literal_type_finalize(GLiteralType *type)
{
    G_OBJECT_CLASS(g_literal_type_parent_class)->finalize(G_OBJECT(type));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : orig  = type d'origine instancié.                            *
*                value = valeur de l'instanciation.                           *
*                                                                             *
*  Description : Crée une représentation de type instancié avec une valeur.   *
*                                                                             *
*  Retour      : Adresse de la structure mise en place.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDataType *g_literal_type_new(GDataType *orig, const literal_value *value)
{
    GLiteralType *result;                   /* Structure à retourner       */

    if (!G_IS_BASIC_TYPE(orig))
        result = NULL;

    else
    {
        result = g_object_new(G_TYPE_LITERAL_TYPE, NULL);

        result->orig = orig;
        result->value = *value;

    }

    return (result != NULL ? G_DATA_TYPE(result) : NULL);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = type à dupliquer.                                     *
*                                                                             *
*  Description : Crée un copie d'un type existant.                            *
*                                                                             *
*  Retour      : Nouvelle instance de type identique à celle fournie.         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static GDataType *g_literal_type_dup(const GLiteralType *type)
{
    GDataType *result;                      /* Copie à retourner           */
    GDataType *orig;                        /* Copie du type interne       */

    orig = g_data_type_dup(type->orig);

    result = g_literal_type_new(orig, &type->value);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type    = type à convertir.                                  *
*                include = doit-on inclure les espaces de noms ?              *
*                                                                             *
*  Description : Décrit le type fourni sous forme de caractères.              *
*                                                                             *
*  Retour      : Chaîne à libérer de la mémoire après usage.                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static char *g_literal_type_to_string(const GLiteralType *type, bool include)
{
    char *result;                           /* Valeur à renvoyer           */

    switch (g_basic_type_get_base(G_BASIC_TYPE(type->orig)))
    {
        case BTP_BOOL:
            result = strdup(type->value.int_val ? "true" : "false");
            break;

        case BTP_INT:
        default:
            asprintf(&result, "%d", type->value.int_val);
            break;

        case BTP_FLOAT:
            asprintf(&result, "%f", type->value.float_val);
            break;

    }

    return result;

}
